/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of this code is David Baum.
 * Portions created by David Baum are Copyright (C) 1998 David Baum.
 * All Rights Reserved.
 */

#include <string.h>
#include <ctype.h>
#include "PHashTable.h"

int PDefaultHash::Hash(const char *key) {
	register unsigned long h=0,g;
	register const char *p;
	
	for(p=key; *p != 0; p++)
	{
		h = (h <<4) + (*p);
		if ((g= h & 0xf0000000) != 0)
		{
			h = h ^ (g >> 24);
			h = h ^ g;
		}
	}
	
	return (int)h;
}

bool PDefaultHash::EqualKey(const char *key1, const char *key2) {
	return strcmp(key1, key2)==0 ? true : false;
}


int PCaselessHash::Hash(const char *key) {
	register unsigned long h=0,g;
	register const char *p;
	
	for(p=key; *p != 0; p++)
	{
		char c = tolower(*p);
		
		h = (h <<4) + (c);
		if ((g= h & 0xf0000000) != 0)
		{
			h = h ^ (g >> 24);
			h = h ^ g;
		}
	}
	
	return (int)h;	
}


bool PCaselessHash::EqualKey(const char *key1, const char *key2) {
	while(*key1) {
		char c1 = *key1++;
		char c2 = *key2++;
		
		if (tolower(c1) != tolower(c2)) return false;
	}
	
	return *key2==0;
}


PHashable::~PHashable() {
}
