/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of this code is David Baum.
 * Portions created by David Baum are Copyright (C) 1998 David Baum.
 * All Rights Reserved.
 */

#ifndef __PHashTable_h
#define __PHashTable_h

#ifndef __PListS_h
#include "PListS.h"
#endif

#ifndef __PTypes_h
#include "PTypes.h"
#endif


class PHashable : public PLinkS<PHashable> {
public:
	virtual	~PHashable();

	const char*	GetKey() const { return fKey; }

protected:
	char*	fKey;
};


class PDefaultHash {
public:
	static int	Hash(const char *key);
	static bool	EqualKey(const char *key1, const char *key2);
};


class PCaselessHash {
public:
	static int	Hash(const char *key);
	static bool	EqualKey(const char *key1, const char *key2);
};


template <class T, class H=PDefaultHash> class PHashTable
{
public:
				PHashTable(int size);
				~PHashTable()	{ delete [] fBuckets; }

	int			GetBucketCount() const	{ return fSize; }
	T*			GetBucket(int i)	{ return fBuckets[i].GetHead(); }

	void		Add(T *item);
	bool		Remove(T *item);
	T*			Find(const char *key);

	void		DeleteAll();
	
	int			Hash(const char *key) const	{ return H::Hash(key) % fSize; }
	
private:
	PListSS<T>*	fBuckets;
	int			fSize;	
};


template <class T, class H> PHashTable<T,H>::PHashTable(int size) {
	fBuckets = new PListSS<T>[size];
	fSize = size;
}


template <class T, class H> T* PHashTable<T,H>::Find(const char *key) {
	T *ptr;
	int hash = Hash(key);
	
	for(ptr=fBuckets[hash].GetHead(); ptr; ptr=(T*)ptr->GetNext()) {
		if (H::EqualKey(ptr->GetKey(), key)) return ptr;
	}

	return nil;
}


template <class T, class H> void PHashTable<T,H>::Add(T *item) {
	int hash = Hash(item->GetKey());
	
	fBuckets[hash].InsertHead(item);	
}


template <class T, class H> void PHashTable<T,H>::DeleteAll() {
	int i;
	T *item;
		
	for(i=0; i<fSize; i++) {
		while((item = fBuckets[i].RemoveHead()) != nil)
			delete item;
	}
}

template <class T, class H> bool PHashTable<T,H>::Remove(T* item) {
	int hash = Hash(item->GetKey());

	return fBuckets[hash].Remove(item);
}


#endif
