/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of this code is David Baum.
 * Portions created by David Baum are Copyright (C) 2000 David Baum.
 * All Rights Reserved.
 */

#include "ShiftExpr.h"
#include "Bytecode.h"
#include "RCX_Cmd.h"


Expr* ShiftExpr::Clone(Mapping *b) const
{
	return new ShiftExpr(Get(0)->Clone(b), Get(1)->Clone(b), fDirection);
}


bool ShiftExpr::Evaluate(int &value) const
{
	int v1, v2;
	
	if (!Get(0)->Evaluate(v1)) return false;
	if (!Get(1)->Evaluate(v2)) return false;
	
	if (fDirection == kRight)
		value = v1 >> v2;
	else
		value = v1 << v2;

	return true;
}


RCX_Value ShiftExpr::EmitAny_(Bytecode &b) const
{
	int dst;
	
	dst = GetTempVar(b);
	if (dst < 0)
		return kIllegalEA;
	
	if (!EmitTo_(b, dst))
		return kIllegalEA;
	
	return RCX_VALUE(kRCX_VariableType, dst);
}


bool ShiftExpr::EmitTo_(Bytecode &b, int dst) const
{
	int value;

	// Get(1) *should* be a constant, but we check anyway
	if (!Get(1)->Evaluate(value)) return false;

	// compute shift value
	if (value > 15)
	{
		// too big...just zero out dst
		b.AddMove(dst, RCX_VALUE(kRCX_ConstantType, 0));
		return true;
	}
	else if (value < 0)
	{
		// don't do anything
		value = 1;
	}
	else
	{
		// value is 2 ^ value
		value = 1 << value;
	}


	// evaluate Get(0) into dst
	if (!Get(0)->EmitTo(b,dst)) return false;
	
	// now multiply/divide
	RCX_Cmd cmd;
	
	cmd.MakeVar( (fDirection==kRight) ? kRCX_DivVar : kRCX_MulVar, dst, RCX_VALUE(kRCX_ConstantType, value));
	b.Add(cmd);

	return true;
}
